-- Copyright 2024 DJ WoodZ

-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:

-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.

---Event for sending settings update broadcast
EnhancedEconomySettingsBroadcastEvent = {}
local EnhancedEconomySettingsBroadcastEvent_mt = Class(EnhancedEconomySettingsBroadcastEvent, Event)
InitEventClass(EnhancedEconomySettingsBroadcastEvent, 'EnhancedEconomySettingsBroadcastEvent');

---Create instance of Event class
-- @return table self instance of class event
function EnhancedEconomySettingsBroadcastEvent.emptyNew()
  local self = Event.new(EnhancedEconomySettingsBroadcastEvent_mt)
  return self
end

---Create new instance of event
-- @param table enhancedEconomySettings the economy settings
function EnhancedEconomySettingsBroadcastEvent.new(enhancedEconomySettings)
  local self = EnhancedEconomySettingsBroadcastEvent.emptyNew()
  self.enhancedEconomySettings = enhancedEconomySettings
  return self
end

---Called on sending side
-- @param integer streamId streamId
-- @param integer connection connection
function EnhancedEconomySettingsBroadcastEvent:writeStream(streamId, connection)
  for fillType, value in pairs(self.enhancedEconomySettings) do
    streamWriteString(streamId, fillType)
    streamWriteFloat32(streamId, value)
  end
end

---Called on receiving side
-- @param integer streamId streamId
-- @param integer connection connection
function EnhancedEconomySettingsBroadcastEvent:readStream(streamId, connection)
  -- If on client
  if g_server == nil then
    self.enhancedEconomySettings = {};

    while true do
      local fillType = streamReadString(streamId);
      local value = streamReadFloat32(streamId);

      -- if fillType is valid
      if (fillType ~= nil) then
        self.enhancedEconomySettings[fillType] = value;
      else
        break
      end
    end

    self:run(connection)
  end
end

---Run action on receiving side
-- @param integer connection connection
function EnhancedEconomySettingsBroadcastEvent:run(connection)
  -- if on server
  if g_server ~= nil then
    -- broadcast to all clients
    g_server:broadcastEvent(EnhancedEconomySettingsBroadcastEvent.new(self.enhancedEconomySettings), false)
  else
    -- if on client
    if (self.enhancedEconomySettings ~= nil) then
      -- update economy
      EnhancedEconomySettings:updateEconomy(self.enhancedEconomySettings, true)
    end
  end
end

---To be called on server. Broadcast settings to all clients.
-- @param table enhancedEconomySettings the economy settings
function EnhancedEconomySettingsBroadcastEvent:sendEvent(enhancedEconomySettings)
  -- if on server
  if g_server ~= nil then
    g_client:getServerConnection():sendEvent(EnhancedEconomySettingsBroadcastEvent.new(enhancedEconomySettings))
  end
end
